# (C) 2004-2006 Veit Wahlich <cru |at| zodia |dot| de>
# Released under the terms of the General Public License.
# Part of the Perl Script Pages. See POD for more information.

package PSP::CGI::Generate;
our $VERSION='0.7';

use warnings;
use strict;
use Exporter;
use PSP::CGI::Transform;

our(@ISA,@EXPORT);
@ISA=qw(Exporter);
@EXPORT=qw(generateGet generateForm);

=head1 NAME

PSP::CGI::Generate - exports code generation functions commonly used in
CGI programs.


=head1 SYNOPSIS

	my $string = generateForm($hash,\%hash,
			{
				key1 => 'value in anonymous hash',
				key2 => 'another value!'
			}
		);
	
	my $string = generateForm(%hash);
	
	my $string = generateForm(
			key1 => 'value from a common hash',
			key2 => 'another value!'
		);
	
	my $string = generateGet($hash,\%hash,
			{
				key1 => 'value in anonymous hash',
				key2 => 'another value!'
			}
		);
	
	my $string = generateGet(%hash);
	
	my $string = generateGet(
			key1 => 'value from a common hash',
			key2 => 'another value!'
		);


=head1 DESCRIPTION

This module exports few functions that generate XHTML code and HTTP request
strings from Perl data structures.
This module was creates as a companion to the Perl Script Pages Engine.
See L<PSP::Engine> for more information.


=head1 OVERVIEW


=head2 Exported Functions

=over

=item generateForm

Generates a XHTML 1.1 compliant <input type="hidden"> tags from either one or
more references to hashs or anonymous hashs or from a single regular hash.
See L</"generateForm function"> for details.

=item generateGet

Generates a HTTP GET string from either one or more references to hashs or
anonymous hashs or from a single regular hash.
See L</"generateGet function"> for details.

=back


=head1 SYNTAX


=head2 generateForm Function

Generates a XHTML 1.1 compliant <input type="hidden"> tags from either one or
more references to hashs or anonymous hashs.

	my $string = generateForm($hash,\%hash,
			{
				key1 => 'value in anonymous hash',
				key2 => 'another value!'
			}
		);

or from a single regular hash:

	my $string = generateForm(%hash);
	
	my $string = generateForm(
			key1 => 'value from a common hash',
			key2 => 'another value!'
		);

The last example will generate the following XHTML code:
"C<E<lt>input type="hidden" name="key1" value="value from a common hash" /E<gt>E<lt>input type="hidden" name="key2" value="another value!" /E<gt>>"

The keys and values are automatically passed through the
L<htmlize function|PSP::CGI::Transform/"htmlize function"> from
L<PSP::CGI::Transform|PSP::CGI::Transform>, so you do not need to take care
for quoting or XHTML compliance in your input data.

This function is exported automatically.

=head3 Required Parameters

=over

=item $hash or \%hash

One or more references to hashs or an anonymous hashs to create the XHTML code
from.

=item %hash

A common hash to create XHTML code from.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $string

Returns a text string containing the resulting XHTML code.

=back

=cut
sub generateForm{
	my @hashes;
	my $string='';
	my($key,$ref);
	if(ref($_[0]) eq 'HASH'){
		@hashes=@_;
	}
	else{
		@hashes={@_};
	}
	foreach $ref (@hashes){
		foreach $key (keys(%{$ref})){
			$string.='<input type="hidden" name="'.htmlize($key).'" value="'.htmlize($ref->{$key}).'" />';
		}
	}
	return($string);
}


=head2 generateGet Function

Generates a HTTP GET string from either one or more references to hashs or
anonymous hashs.

	my $string = generateGet($hash,\%hash,
			{
				key1 => 'value in anonymous hash',
				key2 => 'another value!'
			}
		);

or from a single regular hash:

	my $string = generateGet(%hash);
	
	my $string = generateGet(
			key1 => 'value from a common hash',
			key2 => 'another value!'
		);

The last example will generate the following HTTP GET string:
"C<key1=value+from+a+common+hash&key2=another+value%21>"

As the return string may contain "&" characters, please remember to pass it
through the L<htmlize function|PSP::CGI::Transform/"htmlize function"> from
L<PSP::CGI::Transform|PSP::CGI::Transform> or use the PrintSecureOpenTag if
you want to print the resulting data into (X)HTML code.

This function is exported automatically.

=head3 Required Parameters

=over

=item $hash or \%hash

One or more references to hashs or an anonymous hashs to create the HTTP GET
string from.

=item %hash

A common hash to create HTTP GET string from.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $string

Returns a text string containing the resulting HTTP GET string.

=back

=cut
sub generateGet{
	my @hashes;
	my @list;
	my $value;
	my $ref;
	if(ref($_[0]) eq 'HASH'){
		@hashes=@_;
	}
	else{
		@hashes={@_};
	}
	foreach $ref(@hashes){
		foreach(keys(%{$ref})){
			$value=$ref->{$_};
#			s/\"/\\\"/g;
#			$value=~s/\"/\\\"/g;
			push(@list,encodeHttp($_).'='.encodeHttp($value));
		}
	}
	return(join('&',@list));
}

1;


=head1 AUTHOR

Veit Wahlich

E-Mail: cru |at| zodia |dot| de

WWW: L<http://home.ircnet.de/cru/>


=head1 VERSION

v0.7
Wednesday, 18 January 2006


=head1 COPYRIGHT/LICENSE

Copyright 2004-2006 Veit Wahlich

This software is distributed as free (libre) software under the terms of the
GNU General Public License, version 2 E<lt>L<http://www.gnu.org/copyleft/gpl.html>E<gt>.
The author disclaims responsibility of any damage or harm caused directly or
indirectly by usage of this software. Use only at your own risk.
