# (C) 2004-2006 Veit Wahlich <cru |at| zodia |dot| de>
# Released under the terms of the General Public License.
# Part of the Perl Script Pages. See POD for more information.

package PSP::CGI::Transform;
our $VERSION='0.7';

use warnings;
use strict;
use Exporter;

our(@ISA,@EXPORT);
@ISA=qw(Exporter);
@EXPORT=qw(encodeHttp decodeHttp htmlize htmlizeIsoUmlauts);

=head1 NAME

PSP::CGI::Transform - provides text string transformations commonly used by
CGI programs.

=head1 SYNOPSIS

	use PSP::CGI::Transform;
	my $encoded = encodeHttp($text);
	my $text = decodeHttp($encoded);
	my $htmled = htmlize($text);
	my $htmled = htmlizeIsoUmlauts($isoText);


=head1 DESCRIPTION

This module exports some text string transformation procedures usually needed
by CGI programs.
This module was created as a companion to the Perl Script Pages Engine.
See L<PSP::Engine> for more information.


=head1 OVERVIEW


=head2 Exported Functions

=over

=item encodeHttp

Encodes a generic text string into a HTTP URI encoded string.
See L</"encodeHttp function"> for details.

=item decodeHttp

Decodes a HTTP URI encoded string into a common text string.
See L</"decodeHttp function"> for details.

=item htmlize

Transforms any text into a safe HTML/XHTML representation that needs no
further sanitizing, escaping or quoting for output on HTML/XHTML pages.
See L</"htmlize function"> for details.

=item htmlizeIsoUmlauts

Transforms ISO8859-{1,15} encoded German umlauts in the given text string into
the HTML/XHTML equivalents.
See L</"htmlizeIsoUmlauts function"> for details.

=back


=head1 SYNTAX


=head2 encodeHttp Function

Encodes a generic text string into a HTTP URI encoded string.

	use PSP::CGI::Transform;
	
	my $encoded = encodeHttp($text);

This function is exported automatically.

=head3 Required Parameters

=over

=item $text

The text string to encode.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $encoded

The HTTP URI encoded representation of the input string or undef if $text
was not defined.

=back

=cut
sub encodeHttp{
	my $line=shift;
	if(defined($line)){
		$line=~s/([^a-z0-9 \-_\.\/])/sprintf('%%%02x',ord($1))/egi;
		$line=~s/ /+/g;
	}
	return($line);
}


=head2 decodeHttp Function

Decodes a HTTP URI encoded string into a common text string.

	use PSP::CGI::Transform;
	
	my $text = decodeHttp($encoded);

This function is exported automatically.

=head3 Required Parameters

=over

=item $encoded

The HTTP URI text string to decode. 

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $text

The common text representation of the encoded input string or undef if $encoded
was not defined.

=back

=cut
sub decodeHttp{
	my $line=shift;
	if(defined($line)){
		$line=~tr/+/ /;
		$line=~s/%([a-f0-9]{2})/pack('C',hex($1))/egi;
	}
	return($line);
}


=head2 htmlize Function

Transforms any text into a safe HTML/XHTML representation that needs no
further sanitizing, escaping or quoting for output on HTML/XHTML pages.

	use PSP::CGI::Transform;
	
	my $htmled = htmlize($text);

This function is exported automatically.

=head3 Required Parameters

=over

=item $text

Any text string to transform.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $htmled

The HTML/XHTML safe representation of the input string or undef if $text was
not defined.

=back

=cut
sub htmlize{
	my $text=shift;
	if(defined($text)){
		$text=~s/\&/\&amp\;/g;
		$text=~s/\"/\&quot\;/g;
		$text=~s/\</\&lt\;/g;
		$text=~s/\>/\&gt\;/g;
#		$text=~s/\n/\<br \/>/g;
	}
	return($text);
}


=head2 htmlizeIsoUmlauts Function

Transforms ISO8859-{1,15} encoded German umlauts in the given text string into
the HTML/XHTML equivalents.

	use PSP::CGI::Transform;
	
	my $htmled = htmlizeIsoUmlauts($isoText);

This function does NOT pass the text through the L</"htmlize function">!
You need to do that manually.

This function is exported automatically.

=head3 Required Parameters

=over

=item $text

Any text string to transform.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $htmled

The same text as the input, but all ISO8859-{1,15} German umlauts replaced by
the HTML/XHTML equivalence strings or undef if $text was not defined.

=back

=cut
sub htmlizeIsoUmlauts{
	my $text=shift;
	if(defined($text)){
		$text=~s/\xC4/\&Auml\;/g;
		$text=~s/\xD6/\&Ouml\;/g;
		$text=~s/\xDC/\&Uuml\;/g;
		$text=~s/\xE4/\&auml\;/g;
		$text=~s/\xF6/\&ouml\;/g;
		$text=~s/\xFC/\&uuml\;/g;
		$text=~s/\xDF/\&szlig\;/g;
	}
	return($text);
}

1;


=head1 AUTHOR

Veit Wahlich

E-Mail: cru |at| zodia |dot| de

WWW: L<http://home.ircnet.de/cru/>


=head1 VERSION

v0.7
Wednesday, 18 January 2006


=head1 COPYRIGHT/LICENSE

Copyright 2004-2006 Veit Wahlich

This software is distributed as free (libre) software under the terms of the
GNU General Public License, version 2 E<lt>L<http://www.gnu.org/copyleft/gpl.html>E<gt>.
The author disclaims responsibility of any damage or harm caused directly or
indirectly by usage of this software. Use only at your own risk.
