# (C) 2004-2006 Veit Wahlich <cru |at| zodia |dot| de>
# Released under the terms of the General Public License.
# Part of the Perl Script Pages. See POD for more information.

package PSP::Engine;
our $VERSION='0.7';

use warnings;
use strict;
use PSP::Config;
use Cwd;

=head1 NAME

PSP::Engine - $psp object class and primary engine of the Perl Script Pages


=head1 SYNOPSIS

	use PSP::Engine;
	my $psp = new PSP::Engine(
			conf    => new PSP::Config(),
			cgi     => new PSP::CGI(),
			no_cgi  => 1,
			file    => 'file.psp',
			execute => \$text_string,
			args    => $args
		);
	
	my $success = $psp->execute(\$text_string,'filename',$args);
	
	my $success = $psp->file('file.psp',$args);
	
	$psp->print($text);
	
	$psp->printError($text,$filename);
	
	my $cgi=$psp->cgi();
	
	$value = $psp->setvar('variable',$value);
	my $value = $psp->var('variable');
	
	$psp->var->{variable} = $value;
	my $value = $psp->var->{variable};
	
	my $var = $psp->var();
	$var->{variable} = $value;
	my $value = $var->{variable};
	
	my @vars = $psp->vars();
	
	my $text = $psp->flush();
	
	my $errors = $psp->flushErrors();


=head1 DESCRIPTION

This module provides the engine for the Perl Script Pages (PSP).
PSP code is executed within a PSP::Engine object and the object exports itself
to the PSP code being executed.


=head1 OVERVIEW


=head2 Object Constructors

=over

=item new

Creates a new object of the PSP::Engine class.
The new constructor is the only object constructor for this module.
See L</"new object constructor"> for details.

=back


=head2 Object Methods

=over

=item execute

Executes PSP code from a string reference.
See L</"execute method"> for details.

=item file

Loads PSP code directly from a file, changes directory to the file's
directory and executes the code.
See L</"file method"> for details.

=item print

Appends data to the PSP output.
See L</"print method"> for details.

=item printError

Appends an error message to the PSP error output.
See L</"printError method"> for details.

=item cgi

Returns a reference to the PSP::CGI object associated to the object, if present.
See L</"cgi method"> and L<PSP::CGI> for details.

=item setvar

Stores a value for a given variable name in the object.
This is used for exchanging data between PSP code instances.
See L</"setvar method"> for details.

=item var

Either returns the value stored for a named variable inside the object, or a
reference to the variables hash if variable name is omitted.
See L</"var method"> for details.

=item vars

Returns an array of variable names in use.
See L</"vars method"> for details.

=item flush

Returns the object's output data and resets the data storage.
See L</"flush method"> for details.

=item flushErrors

Returns the object's error messages and resets the error message storage.
See L</"flushErrors method"> for details.

=back


=head1 SYNTAX


=head2 new Object Constructor

Creates a new object of the PSP::Engine class.

	use PSP::Engine;
	my $psp = new PSP::Engine(
			conf    => new PSP::Config(),
			cgi     => new PSP::CGI(),
			no_cgi  => 1,
			file    => 'file.psp',
			execute => \$text_string,
			args    => $args
		);

=head3 Required Parameters

None.

=head3 Optional Parameters

The following parameters are hash key/value parameters.

=over

=item conf

May be an object of L<PSP::Config|PSP::Config>. If omitted, a new object will be created.

=item cgi

May be an object of L<PSP::CGI|PSP::CGI>. If omitted, a new object will be created.

=item no_cgi

If set to a true value and cgi parameter is omitted, no L<PSP::CGI|PSP::CGI> object will be created.

=item file

May be a PSP file to parse and execute.
See the L</"file method"> for details.

=item execute

May be a reference to a test string containing PSP code to parse and execute.
execute is ignored if the file parameter is given.
See the L</"execute method"> for details.

=item args

May be a reference to i.e. an array of arguments to pass to a PSP string or file
that is executed.
See L</"file method"> or L</"execute method"> for details.

=back

=cut
sub new{
	my $class=shift;
	my %hash=@_;
	my $self={
			var => {},
			output => '',
			errors => ''
		};
	my $args=undef;
	bless($self,$class);
	if(exists($hash{conf})){
		$self->{conf}=$hash{conf};
	}
	else{
		$self->{conf}=new PSP::Config();
	}
	if(exists($hash{cgi})){
		$self->{cgi}=$hash{cgi};
	}
	elsif(not(exists($hash{no_cgi}) && $hash{no_cgi})){
		require PSP::CGI;
		$self->{cgi}=new PSP::CGI;
	}
	if(exists($hash{args})){
		$args=$hash{args};
	}
	if(exists($hash{file})){
		$self->file($hash{file},$args);
	}
	elsif(exists($hash{execute})){
		$self->execute($hash{execute},undef,$args);
	}
	return($self);
}


=head2 execute Method

Executes PSP code from a string reference.

	my $success = $psp->execute(\$text_string,$filename,$args);

=head3 Required Parameters

=over

=item \$text_string

A reference to a text string containing PSP code.

=back

=head3 Optional Parameters

=over

=item $filename

A text string containing a filename for the code being executed.
It is only used for constructing an error message if execution fails and may
be undef.

=item $args

If set, it will be available to the PSP code executed via the $args variable.
It is intended to supply arguments to the code, i.e. via an array or hash
reference. For example $psp->execute(\$data,[$a,$b]) will execute the PSP code
stored in $data. The code can access the first argument ($a) via $args->[0] and
the second one ($b) via $args->[1].

=back

=head3 Return Values

=over

=item $success

The execute method returns either 1 or undef to declare success or failure of
code execution.

=back

=cut
sub execute{
	my $psp=shift;
	my $cgi=$psp->{cgi};
	my $var=$psp->{var};
	my $data=shift || return(undef);
	my $file=shift || undef;
	my $args=shift;
#	my $function=(defined($file))?('file_'.$file):('unnamed');
#	$function=~s/[^a-z0-9]/_/ig;
	$psp->perlify($data);
#	eval('my $psp=$psp; my $cgi=$cgi; sub '.$function.'{'.${$data}.'} &'.$function.'(@{$args})');
	eval(${$data});
	if($@){
		$@=~s/ \(eval [0-9]+\)//;
		$psp->printError("Error executing code: ".$@."\n",$file);
		return(undef);
	}
	return(1);
}


=head2 file Method

Loads PSP code directly from a file, changes directory to the file's
directory and executes the code.

	my $success = $psp->file('file.psp',$args);

=head3 Required Parameters

=over

=item $file

A text string containing a filename to load the PSP code from for being
executed.

Please note that paths/filenames with leading slash ('/') are not absolute to
the filesystem but relative to the BaseDirectory, configured through
PSP::Config!
You may set BaseDirectory to '' if you wish to have "real" absolute paths.

=back

=head3 Optional Parameters

=over

=item $args

If set, it will be available to the PSP code executed via the $args variable.
It is intended to supply arguments to the code, i.e. via an array or hash
reference. For example $psp->file('file.psp',[$a,$b]) will execute file.psp,
the code within file.psp can access the first argument ($a) via $args->[0] and
the second one ($b) via $args->[1].

=back

=head3 Return Values

=over

=item $success

The file method returns either 1 or undef to declare success or failure of
code execution.

=back

=cut
sub file{
	my $self=shift;
	my $file=shift || return(undef);
	my $args=shift;
	my $cwd=cwd();
	my $data=$self->execute($self->loadFile($file),$file,$args);
	chdir($cwd);
	return($data);
}


=head2 print Method

Appends data to the PSP output.

	$psp->print($text);
	$psp->print(@text);

=head3 Required Parameters

=over

=item $text or @text

One or more text strings to append to PSP output.

=back

=head3 Optional Parameters

None.

=head3 Return Values

None.

=cut
sub print{
	my $self=shift;
	$self->{output}.=join('',@_);
}


=head2 printError Method

Appends an error message to the PSP error output.

	$psp->printError($text,$filename);

At the moment, errors printed through this method are also appended to the PSP
output. This may change (i.e. become optional) in future versions.

=head3 Required Parameters

=over

=item $text

Text string to append to PSP error messages.

=back

=head3 Optional Parameters

=over

=item $filename

If given, the error message will be prepended with this filename.

=back

=head3 Return Values

None.

=cut
sub printError{
	my $self=shift;
	my $text=shift;
	my $file=shift;
	$self->{output}.=((defined($file))?($file.': '):('')).$text;
	$self->{errors}.=((defined($file))?($file.': '):('')).$text;
}


=head2 cgi Method

Returns a reference to the PSP::CGI object associated to the object, if present.

	my $cgi = psp$->cgi();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $cgi

A reference to the L<PSP::CGI|PSP::CGI> object associated to the object, if present.
See L<PSP::CGI> for details.

=back

=cut
sub cgi{
	my $self=shift;
	return($self->{cgi});
}


=head2 setvar Method

Stores a value for a given variable name in the object. This is used for exchanging data between PSP code instances.

	my $varvalue = psp$->setvar($varname,$varvalue);

=head3 Required Parameters

=over

=item $varname

The name used to identify the variable.

=item $varvalue

The value to set the variable given above to.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $varvalue

A copy of the $varvalue parameter.

=back

=cut
sub setvar{
	my $self=shift;
	my $name=shift;
	my $value=shift;
	if(defined($name)){
		return($self->{var}->{$name}=$value);
	}
	else{
		return(undef);
	}
}


=head2 var Method

Either returns the value stored for a named variable inside the object,
or a reference to the variables hash if variable name is omitted.

	my $varvalue = psp$->var($varname);

	my $var = psp$->var();
	$var->{$varname} = $varvalue;
	my $varvalue = $var->{$varname};

=head3 Required Parameters

None.

=head3 Optional Parameters

=over

=item $varname

The name used to identify the variable.

=back

=head3 Return Values

=over

=item $varvalue

The value stored in the object for a named variable, undef if no such variable
exists.

=item $var

A reference to the variables hash if $varname was omitted/undef.

=back

=cut
sub var{
	my $self=shift;
	my $name=shift;
	if(defined($name)){
		return($self->{var}->{$name});
	}
	else{
		return($self->{var});
	}
}


=head2 vars Method

Returns an array of variable names in use.

	my @vars = $psp->vars();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item @vars

Array of variable names/hash search keys used to store data.

=back

=cut
sub vars{
	my $self=shift;
	return(keys(%{$self->{var}}));
}

=head2 flush Method

Returns the object's output data and resets the data storage.

	my $text = $psp->flush();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $text

The value of the output data queue.

=back

=cut
sub flush{
	my $self=shift;
	my $output=$self->{output};
	$self->{output}='';
	return($output);
}


=head2 flushErrors Method

Returns the object's error messages and resets the error message storage.

	my $errros = $psp->flushErrors();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $errors

The value of the error messages queue.

=back

=cut
sub flushErrors{
	my $self=shift;
	my $errors=$self->{errors};
	$self->{errors}='';
	return($errors);
}

# Internal object method:
sub loadFile{
	my $self=shift;
	my $file=shift;
	my $conf=$self->{conf};
	my $out='';
	my $in;
	my $dir='.';
	my $newfile;
	if($file=~/\//){
		($dir,$file)=($file=~/^(.*\/)(.*?)$/);
	}
	(chdir(($dir=~/^\//)?($conf->{BaseDirectory}.$dir):($dir)) && open($in,'<',$file))
		|| do{
			$self->printError('Error opening file for reading: '.$!."\n",(($dir=~/^\//)?('[Base Directory]'.$dir):($dir)).$file);
			return(undef);
		};
	while(readline($in)){
		$out.=$_;
	}
	close($in);
	return(\$out);
}

# Internal function:
sub textToPrint{
	my $data=shift;
	return(undef) unless(defined($data));
	# substitute ' characters
	$data=~s/\'/\'."\'".\'/g;
	return(";\$psp->{output}.='$data';");
}

# Internal object method:
sub perlify{
	my $self=shift;
	my $data=shift || return(undef);
	my $conf=$self->{conf};
	# ensure at least one perl-code area exists
	$$data.=$conf->{OpenTag}.$conf->{CloseTag};
	# substitute print tags
	$$data=~s/$conf->{PrintOpenTag}(.*?)$conf->{CloseTag}/$conf->{OpenTag};\$psp->{output}.=$1;$conf->{CloseTag}/sg;
	# substitute secure print tags
	if(exists($self->{cgi}) && defined($self->{cgi})){
		$$data=~s/$conf->{PrintSecureOpenTag}(.*?)$conf->{CloseTag}/$conf->{OpenTag};\$psp->{output}.=PSP::CGI::htmlize($1);$conf->{CloseTag}/sg;
	}
	# substitute "inner" non-perl areas to print
	$$data=~s/$conf->{CloseTag}(.*?)$conf->{OpenTag}/defined($1) && textToPrint($1)/seg;
	# substitute leading and training non-perl areas to print
	$$data=~s/^(.*?)$conf->{OpenTag}/defined($1) && textToPrint($1)/se;
	$$data=~s/$conf->{CloseTag}(.*?)$/defined($1) && textToPrint($1)/se;
	return($data);
}

1;

=head1 USAGE INSTRUCTIONS

=head2 Object Construction

When constructing a PSP::Engine object, it expects a PSP::Config and a
PSP::CGI object as hash parameters:

	my $psp = new PSP::Engine(
			conf    => new PSP::Config(),
			cgi     => new PSP::CGI()
		);

If one or both objects are omitted, the "missing" object(s) will be created.

See also L<PSP::Config> and L<PSP::CGI>.

The no_cgi => 1 option prevents the PSP::Engine constructor from creating a new
PSP::CGI object:

	my $psp = new PSP::Engine(
			no_cgi  => 1
		);

This is useful if PSP is not used for CGI but for i.e. templating tasks.

Both the L<"file method"> and the L</"execute method"> may be used directly from the
object constructor for comfort:

	my $psp = new PSP::Engine(
			file    => 'file.psp',
			args    => $args
		);

	my $psp = new PSP::Engine(
			execute => \$text_string,
			args    => $args
		);


=head2 Executing PSP Code

PSP code may either be loaded from a file using the L</"file method"> or directly
from a text string reference using the L</"execute method">:

	my $success = $psp->execute(\$text_string,'filename',$args);
	
	my $success = $psp->file('file.psp',$args);

Both methods return either 1 or undef to declare success or failure of
code execution.

The $args parameter is optional. If set, it will be available to the PSP code
executed via the $args variable. It is intended to supply arguments to the
code, i.e. via an array or hash reference.
For example $psp->file('file.psp',[$a,$b]) will execute file.psp, the code
within file.psp can access the first argument ($a) via $args->[0] and the
second one ($b) via $args->[1].

Please note that paths/filenames with leading slash ('/') are not absolute to
the filesystem but relative to the BaseDirectory, configured through
PSP::Config!
You may set BaseDirectory to '' if you wish to have "real" absolute paths.

The second parameter for the L</"execute method">, 'filename', is optional and may
be undef. It is only used for constructing an error message if execution fails.

You can use the L</"execute method"> and L</"file method"> more than once,
resulting output is appended.


=head2 Printing Data to PSP Output

Using the L</"print method">, you may append data to the PSP output to be flushed
later:

	$psp->print($text);

For error messages, there is a L</"printError method">, also accepting a text
string as parameter:

	$psp->printError($text,$filename);

The $filename parameter is optional and may be omitted. If given, the error
message will be prepended with this filename.

At the moment, errors printed through the L</"printError method"> are also appended
to the PSP output. This may change (i.e. become optional) in future versions.


=head2 Resulting Output

After PSP code execution, the resulting output is to be flushed from the object
to a string using the L</"flush method">:

	my $text = $psp->flush();

The same applies to code execution error messages via the L</"flushErrors method">:

	my $errors = $psp->flushErrors();

The object can be re-used after flushing.

=cut


=head1 AUTHOR

Veit Wahlich

E-Mail: cru |at| zodia |dot| de

WWW: L<http://home.ircnet.de/cru/>


=head1 VERSION

v0.7
Wednesday, 18 January 2006


=head1 COPYRIGHT/LICENSE

Copyright 2004-2006 Veit Wahlich

This software is distributed as free (libre) software under the terms of the
GNU General Public License, version 2 E<lt>L<http://www.gnu.org/copyleft/gpl.html>E<gt>.
The author disclaims responsibility of any damage or harm caused directly or
indirectly by usage of this software. Use only at your own risk.
